/*
Model 819 API include file. Windows
Copyright Sensoray Company Inc. 2012
*/

#ifndef _S819_H_
#define _S819_H_

/*****************************************************
* CONSTANTS
*****************************************************/

//default and max bitrates for H.264
#define DEF_BR_QCIF             500000
#define DEF_BR_CIF              1000000
#define DEF_BR_D1               2000000

#define MIN_BR_MASTER           64000
#define MAX_BR_MASTER           3000000

#define MIN_BR_SUB              32000
#define MAX_BR_SUB              1000000

//JPEG quality setting allowed range
#define MIN_JPG_QUAL            0
#define MAX_JPG_QUAL            4

//Max GOP size
#define GOP_SIZE_MAX            240     

//Resolution constants
//Horizontal
#define HS_PAL_D1               720
#define HS_PAL_CIF              (HS_PAL_D1/2)
#define HS_PAL_QCIF             (HS_PAL_D1/4)

#define HS_NTSC_D1              720
#define HS_NTSC_CIF             (HS_NTSC_D1/2)
#define HS_NTSC_QCIF            (HS_NTSC_D1/4)

//Vertical
#define VS_PAL_D1               576
#define VS_PAL_CIF              (VS_PAL_D1/2)
#define VS_PAL_QCIF             (VS_PAL_D1/4)

#define VS_NTSC_D1              480
#define VS_NTSC_CIF             (VS_NTSC_D1/2)
#define VS_NTSC_QCIF            (VS_NTSC_D1/4)

// Registers & masks for motion detection
#define IND_REG_MVSENS          0xC08
#define CHN_OFFSET_MVSENS       0x20
#define REG_IND_ACCESS          0xB800
#define REG_IND_DATA            0xB804
#define CHN_OFFSET_SIG          0x40


/*****************************************************
* TYPES
*****************************************************/
//Supported resolutions
typedef enum {
    RES_QCIF = 0,
    RES_CIF,
    RES_D1,
    RES_MAX,           //for array sizing only
} S819_RESOLUTION;

//Supported TV standards
typedef enum {
    TVS_PAL,
    TVS_NTSC,
    TVS_MAX,        //for array sizing
} S819_TVSTANDARD;


//Bitrate control modes
typedef enum {
    BITRATE_H264_NO_RC,         //Var with max limit (not recommended)
	BITRATE_H264_CBR,           //constant bitrate (default)
	BITRATE_H264_VBR,           //variable bitrate
    BITRATE_MAX,        		//for array sizing only
} S819_BR_CTRL;

//Capture  mode (video decoder mode)
typedef struct {
    S819_TVSTANDARD     tvs;                // all channels for one board must have the same tv standard
											// in order to change video standard, all channels per board
											// must be stopped
    int                 bright;
    int                 contrast;
    int                 saturation;
    int                 hue;
    int                 sharpness;
	int                 reserved[8];
} CHAN_MODE;

//Stream mode (compression)
typedef struct {
	// common to all formats
    S819_RESOLUTION     resolution;         //
    int                 fps;                //1..MAX (30 or 25)
    
	// for H.264 streams
    int                 gopSize;            //1..?
    S819_BR_CTRL        brCtrl;             //CBR. VBR, ?
    int                 bitrate;            //max bitrate (?)

	int                 reserved[8];
} STREAM_MODE_H264;

//Stream mode (compression)
typedef struct {
	// common to all formats
    S819_RESOLUTION     resolution;         //
    int                 fps;                //1..MAX (30 or 25)
	int                 qp;					// quantization parameter [10..51]
	int                 reserved[8];
} STREAM_MODE_JPEG;

//Stream mode YUV
typedef struct {
	// common to all formats
    S819_RESOLUTION     resolution;         //
    int                 fps;                //1..MAX (30 or 25)
	HWND				hwnd;				// preview window
	int                 reserved[8];
} STREAM_MODE_YUV;




typedef struct {
    int                 ctrl;
} CONTROL;

//Channel structure
typedef struct {
    CHAN_MODE           cMode;          //channel capture mode
    STREAM_MODE_H264    psMode;         //primary H.264 stream
    STREAM_MODE_H264    ssMode;         //secondary H.264 stream
    STREAM_MODE_JPEG    jsMode;         //JPEG stream
    STREAM_MODE_YUV     usMode;         //uncompressed (preview) stream
    CONTROL             ctrl;           //controls some operations, e.g. SetMode (fast or slow)
	int                 reserved[32];
} S819_CHAN;

// Motion detection sensitivity settings
typedef struct {
  BYTE MD_REFFLD;       // updating time of reference field 0-every field (default), 1-use MD_SPEED
  BYTE MD_FIELD;        // Field 0 - odd fields only (default), 1 - even fields only, 2 - any field, 3 - odd and even
  BYTE MD_LVSENS;       // Level sensitivity 0 (Most Senstive, default) .. 15 (least sensitive)
  BYTE MD_CELSENS;      // 
  BYTE MD_SPEED;
  BYTE MD_SPSENS;
  BYTE BD_LVSENS;
  BYTE ND_TMPSENS;
  BYTE ND_LVSENS;
} S819_MD_SETTINGS;

typedef struct {
	unsigned long lTimeStamp;
	unsigned char odddata[32*4];
	unsigned char evendata[32*4];
} S819_MVFLAG_DATA;

//Stream types and masks
typedef enum {
    STREAM_H264_PRI = 0,
    STREAM_H264_SEC,
    STREAM_JPEG,
    STREAM_YUV,
    STREAM_AUDIO, // audio stream  is not started by itself.  This is the return type from waitbuffer only
	              // audio will automatically be started when either H264 stream is started.
    STREAM_MV,      // future use
    STREAM_MAX
} S819_STREAM_TYPE;

#define SMASK(x)        (1<<(x))        //stream type mask

//Capture node handle.
//A capture node is a set of streams grouped together
//in a manner convenient for the application (using
//S819_CreateCnode fucntion).
//E.g. a capture node may consist of all primary
//streams for all channels, or of just one 
//particular stream.

typedef void *         HCNODE;


/*
//Crosspoint switch structure
typedef struct {
    

} S819_XPSW;
*/

// frame types.
#define FTYPE_SPS 1
#define FTYPE_PPS 2
#define FTYPE_IFRAME 3
#define FTYPE_PFRAME 4
#define FTYPE_BFRAME 5  // unused


typedef struct {
    int                 chan;
    S819_STREAM_TYPE    stream;
    unsigned char       *buf;
    int                 length;
    unsigned int        frm;
    unsigned int        pts;
    void                *rsv;
	unsigned int        ftype; // for H.264 only. SPS, PPS, I-frame, P-frame, B-frame
	int                 reserved[8]; // future use
} BUFFER;

// Motion detection callback function signature
typedef void (_stdcall *S819_MVFlagCallBack)(unsigned int nChannel,S819_MVFLAG_DATA *pdata,void *pContext);

/*****************************************************
* ERROR CODES
*****************************************************/
typedef int         ECODE;                  //error code

#define ECODE_OK                0                   //success
#define ECODE_BASE              0                   //base for error codes

#define ECODE_SDK_INIT          (ECODE_BASE + 1)    //SDK initialization error
#define ECODE_SDK_START         (ECODE_BASE + 2)    //SDK start error
#define ECODE_SDK_COUNT         (ECODE_BASE + 3)    //error getting chip count
#define ECODE_NO_CHANS          (ECODE_BASE + 4)    //no 819 boards detected
#define ECODE_ILLEGAL_CHAN      (ECODE_BASE + 5)    //illegal channel number
#define ECODE_SETVD             (ECODE_BASE + 6)    //error setting VD
#define ECODE_CHAN_PARAM        (ECODE_BASE + 700)    //illegal S819_CHAN value
#define ECODE_SETVSP            (ECODE_BASE + 8)    //error setting VS for primary stream
#define ECODE_SETVSS            (ECODE_BASE + 9)    //error setting VS for secondary stream
#define ECODE_INVALID_HCNODE    (ECODE_BASE + 10)   //invalid HCNODE
#define ECODE_REG_CB            (ECODE_BASE + 11)   //error registering callback
#define ECODE_FN_UNDEFINED      (ECODE_BASE + 12)   //function undefined for stream
#define ECODE_INVALID_DATAP     (ECODE_BASE + 13)   //invalid data pointer
#define ECODE_INVALID_DATAB     (ECODE_BASE + 14)   //invalid data buffer
#define ECODE_DATA_TIMEOUT      (ECODE_BASE + 15)   //data timeout
#define ECODE_STRM_ATTACHED     (ECODE_BASE + 16)   //stream already attached to node
#define ECODE_STRM_NOTATTACHED  (ECODE_BASE + 17)   //stream not attached to node
#define ECODE_START_STRM        (ECODE_BASE + 18)   //error starting stream
#define ECODE_STOP_STRM         (ECODE_BASE + 19)   //error stopping stream
#define ECODE_SEMAPHORE         (ECODE_BASE + 20)   //semaphore error
#define ECODE_INVALID_PACKET    (ECODE_BASE + 21)   //invalid data packet
#define ECODE_INVALID_BUFFER    (ECODE_BASE + 22)   //invalid buffer pointer
#define ECODE_INVALID_BFR_DATA  (ECODE_BASE + 23)   //buffer contains invalid data
#define ECODE_RESOLUTION        (ECODE_BASE + 24)   //secondary resolution not lower than primary
#define ECODE_SETIBP            (ECODE_BASE + 25)   //error setting IBP parameters
#define ECODE_SETBRC            (ECODE_BASE + 26)   //error setting bitrate control
#define ECODE_SETBR             (ECODE_BASE + 27)   //error setting bitrate
#define ECODE_SETVP             (ECODE_BASE + 28)   //error setting decoder mode
#define ECODE_I2C               (ECODE_BASE + 29)   //I2C error
#define ECODE_SETJPEGFORMAT     (ECODE_BASE + 30)   //error setting JPEG format
#define ECODE_SETJPEGFPS        (ECODE_BASE + 31)   //error setting JPEG fps
#define ECODE_SETJPEGQUALITY    (ECODE_BASE + 32)   //error setting JPEG quality
#define ECODE_TIMEOUT           (ECODE_BASE + 33)   //error setting JPEG quality

 
/*****************************************************
* API FUNCTIONS
*****************************************************/

#ifdef ISDLLBUILD
    #define MID819_API extern "C" __declspec(dllexport)
#else
#ifdef __cplusplus
    #define MID819_API extern "C" __declspec(dllimport)
#else
    #define MID819_API __declspec(dllimport)
#endif
#endif

//Detects the number of capture channels in the system
MID819_API ECODE __stdcall S819_Enumerate(int *nChans);
//Set channel operating mode. (chan starts at 1)
MID819_API ECODE __stdcall S819_SetMode(int chan, S819_CHAN *s819chan);
//Creates a capture node
MID819_API HCNODE __stdcall S819_CreateCnode();
MID819_API void __stdcall S819_DeleteCnode(HCNODE hCnode);
//Attaches channels/streams to a capture node
MID819_API ECODE __stdcall S819_AttachStreams(HCNODE hCnode, int chan, int streams);
//Detach channel/streams from capture node
MID819_API ECODE __stdcall S819_DetachStreams(HCNODE hCnode);
//Start/stop selected streams
MID819_API ECODE __stdcall S819_StartStreams(HCNODE hCnode, int chan, int streams);
MID819_API ECODE __stdcall S819_StopStreams(HCNODE hCnode, int chan, int streams);
//Start/stop all streams of a CNODE
MID819_API ECODE __stdcall S819_StartAll(HCNODE hCnode);
MID819_API ECODE __stdcall S819_StopAll(HCNODE hCnode);

//Wait for data (blocking)
MID819_API ECODE __stdcall S819_WaitBuffer(HCNODE hCnode, BUFFER *buf);
MID819_API ECODE __stdcall S819_ReleaseBuffer(BUFFER *buf);

//crosspoint switch control
MID819_API ECODE __stdcall S819_SetXPSwitch (int board, int inp, int out);
MID819_API ECODE __stdcall S819_SetXPOut (int board, int mask);

MID819_API ECODE __stdcall S819_StartMVFlag (HCNODE hCnode);
MID819_API ECODE __stdcall S819_StopMVFlag (HCNODE hCnode);
MID819_API ECODE __stdcall S819_RegMVFlagCB (S819_MVFlagCallBack funCallBack, void *pContext);


MID819_API void __stdcall S819_Close (void);

#endif  //_S819_H_


